import _common

class circuitInfo(object):

    def __init__(self):
        self.fields = [
             ['start_time', 0, '开始时间'],
             ['end_time', 0, '结束时间'],
             ['ranking', 0, '排名'],
             ['score', 0, '积分'],
        ]

class rankingInfo(object):

    def __init__(self):
        self.fields = [
            ['account_id', 0, 'account'],
            ['name', 0, '名字'],
            ['head_id', 0, '头像'],
            ['head_frame', 0, '头像框'],
            ['hero_id', 0, '英雄ItemId'],
            ['skin_id', 0, '皮肤ItemId'],
            ['ranking', 0, '排名'],
            ['score', 0, '积分'],
            ['score_boost', 0, '加成的积分'],
            ['cec', 0, 'cec(个人榜信息才有)'],
        ]

class AAMarket(object):

    def __init__(self):
        self.apis = [
            {
                'method': 'POST',
                'name': '/api/market/product/list/:net_id',
                'desc': '获取上架出售的NFTs(瀑布流式api) jwtheader为可选参数',
                'group': '!AAMarket',
                'url': 'https://market-test.kingsome.cn/api/market/product/list/:net_id',
                'headers': _common.MaybeJwtHeader,
                'is_json_params': True,
                'request_params': [
                    [':net_id', 0, '链id'],
                ],
                'params': [
                    ['page_size', 0, '每页大小'],
                    ['cursor', '', '游标 第一页传空'],
                    ['search', [
                        ['name', '', '名字查询 空(所有)'],
                    ], '模糊查询'],
                    ['filter', [
                        ['price_min', '', '价格下限 空(无下限)'],
                        ['price_max', '', '价格上限 空(无上限)'],
                        ['!item_ids', [0], '道具id列表 空(所有)'],
                        ['!hero_ranks', [0], '英雄品阶列表 空(所有)']
                    ], '过滤条件'],
                    ['sort', [
                        ['!fields', [
                            ['name', '', '字段名 目前支持的字段(price)'],
                            ['type', 0, '排序方式 -1:倒序 0:默认排序 1:正序'],
                        ], '排序字段']
                    ], '排序规则'],
                ],
                'response': [
                    _common.RspHead(),
                    ['page', _common.StreamPage(), '分页信息'],
                    ['!rows', [_common.MarketGoods()], '数据'],
                ]
            },
            {
                'method': 'GET',
                'name': '/api/market/product/query_price',
                'desc': '查询在售卖商品价格',
                'group': '!AAMarket',
                'url': 'https://market-test.kingsome.cn/api/market/product/query_price',
                'params': [
                    ['net_id', 0, '链id'],
                    ['contract_address', '', '合约地址'],
                    ['item_id', '', '道具id'],
                    ['quality', '', '道具品质(目前只有Hero生效)'],
                ],
                'response': [
                    _common.RspHead(),
                    ['lowest_price_goods', _common.MarketGoods(), '最低价格商品(如果没则为null)'],
                    ['highest_price_goods', _common.MarketGoods(), '最高价格商品(如果没则为null)'],
                ]
            },
            {
                'method': 'GET',
                'name': '/api/market/product/goods/:net_id/:contract_address/:token_id',
                'desc': '获取单个商品信息',
                'group': '!AAMarket',
                'url': 'https://market-test.kingsome.cn/api/market/product/goods/:net_id/:contract_address/:token_id',
                'params': [
                    [':net_id', 0, '链id'],
                    [':contract_address', '', '合约地址'],
                    [':token_id', '', 'token_id'],
                ],
                'response': [
                    _common.RspHead(),
                    ['data', _common.MarketGoods(), '商品信息(如果没则为null), 如果未上架则data.event==null'],
                ]
            },
            {
                'method': 'GET',
                'name': '/api/market/product/category/:net_id',
                'desc': '获取上架出售的NFTs分类数量',
                'group': '!AAMarket',
                'url': 'https://market-test.kingsome.cn/api/market/product/category/:net_id',
                'params': [
                    [':net_id', 0, '链id'],
                ],
                'response': [
                    _common.RspHead(),
                    ['!data', [
                        ['item_id', 0, '道具id'],
                        ['num', 0, '商品数量'],
                    ], '分类数量'],
                ]
            },
            {
                'name': '/api/market/transaction/history/:net_id/:account_address',
                'desc': '获取账号交易历史',
                'group': '!AAMarket',
                'url': 'https://market-test.kingsome.cn/api/market/transaction/history/:net_id/:account_address',
                'params': [
                    [':net_id', 0, '链id'],
                    [':account_address', '', '钱包地址'],
                    ['type', '', '0:全部 1:买进 2：卖出'],
                    ['page_size', 0, '每页大小'],
                    ['cursor', '', '游标 第一页传空'],
                    ['search_name', '', 'nft名字 空(所有)'],
                ],
                'response': [
                    _common.RspHead(),
                    ['page', _common.StreamPage(), '分页信息'],
                    ['!rows', [_common.SaleGoods()], '数据'],
                ]
            },
            {
                'name': '/api/asset/:net_id/:account_address',
                'desc': '获取账号资产',
                'group': '!AAMarket',
                'url': 'https://market-test.kingsome.cn/api/asset/:net_id/:account_address',
                'params': [
                    [':net_id', 0, '链id'],
                    [':account_address', '', '钱包地址'],
                    ['type', '', '0:全部 1:上架中 2:未上架 3:质押中'],
                    ['page_size', 0, '每页大小'],
                    ['cursor', '', '游标 第一页传空'],
                    ['search_name', '', 'nft名字 空(所有)'],
                ],
                'response': [
                    _common.RspHead(),
                    ['page', _common.StreamPage(), '分页信息'],
                    ['!rows', [_common.AssetNftInfo()], '数据'],
                ]
            },
            {
                'method': 'GET',
                'name': '/api/listing/:net_id/:account_address',
                'desc': '获取自己上链商品',
                'group': '!AAMarket',
                'url': 'https://market-test.kingsome.cn/api/listing/:net_id/:account_address',
                'params': [
                    [':net_id', 0, '链id'],
                    [':account_address', '', '钱包地址'],
                    ['page_size', 0, '每页大小'],
                    ['cursor', '', '游标 第一页传空'],
                    ['search_name', '', 'nft名字 空(所有)'],
                ],
                'response': [
                    _common.RspHead(),
                    ['page', _common.StreamPage(), '分页信息'],
                    ['!data', [_common.MarketGoods()], '数据'],
                ]
            },
            {
                'method': 'GET',
                'name': '/api/shopcart/list',
                'desc': '购物车-商品列表',
                'group': '!AAMarket',
                'url': 'https://market-test.kingsome.cn/api/shopcart/list',
                'headers': _common.JwtHeader,
                'permission': _common.LoginRequired,
                'params': [
                ],
                'response': [
                    _common.RspHead(),
                    ['!data', [_common.MarketGoods()], '数据'],
                ]
            },
            {
                'method': 'POST',
                'name': '/api/shopcart/add',
                'desc': '购物车-添加商品',
                'group': '!AAMarket',
                'url': 'https://market-test.kingsome.cn/api/shopcart/add',
                'headers': _common.JwtHeader,
                'permission': _common.LoginRequired,
                'is_json_params': True,
                'params': [
                    ['net_id', 0, '链id'],
                    ['!tokens', [
                        ['token_id', '', 'token_id'],
                        ['contract_address', '', 'contract_address'],
                    ], 'nft list']
                ],
                'response': [
                    _common.RspHead(),
                ]
            },
            {
                'method': 'POST',
                'name': '/api/shopcart/del',
                'desc': '购物车-删除商品',
                'group': '!AAMarket',
                'url': 'https://market-test.kingsome.cn/api/shopcart/del',
                'headers': _common.JwtHeader,
                'permission': _common.LoginRequired,
                'is_json_params': True,
                'params': [
                    ['net_id', 0, '链id'],
                    ['!tokens', [
                        ['token_id', '', 'token_id'],
                        ['contract_address', '', 'contract_address'],
                    ], 'nft list']
                ],
                'response': [
                    _common.RspHead(),
                ]
            },
            {
                'method': 'POST',
                'name': '/api/shopcart/clear',
                'desc': '购物车-清空购物车',
                'group': '!AAMarket',
                'url': 'https://market-test.kingsome.cn/api/shopcart/clear',
                'headers': _common.JwtHeader,
                'permission': _common.LoginRequired,
                'params': [
                    ['net_id', 0, '链id'],
                ],
                'response': [
                    _common.RspHead(),
                ]
            },
            {
                'method': 'POST',
                'name': '/api/nft/stacking/unlock',
                'desc': '质押-unlock',
                'group': '!AAMarket',
                'url': 'https://market-test.kingsome.cn/api/nft/stacking/unlock',
                'headers': _common.JwtHeader,
                'permission': _common.LoginRequired,
                'is_json_params': True,
                'params': [
                    ['net_id', 0, '链id'],
                    ['contract_address', '', 'contract_address'],
                    ['!tokens', [
                        ['token_id', '', 'token_id'],
                    ], 'nft list(目前只支持英雄并且只支持一个)']
                ],
                'response': [
                    _common.RspHead(),
                    ['trans_id', '', '事务id'],
                    ['trans_req', _common.MFTransactionRequest(), '调用合约参数'],
                ]
            },
            {
                'method': 'POST',
                'name': '/api/ingame/asset/hero/list',
                'desc': '获取游戏内资产-英雄',
                'group': '!AAMarket',
                'url': 'https://market-test.kingsome.cn/api/ingame/asset/hero/list',
                'headers': _common.JwtHeader,
                'is_json_params': True,
                'request_params': [
                ],
                'params': [
                    ['page_size', 0, '每页大小'],
                    ['cursor', '', '游标 第一页传空'],
                    ['search', [
                        ['name', '', '名字查询 空(所有)'],
                    ], '模糊查询'],
                    ['filter', [
                        ['!item_ids', [0], '道具id列表 空(所有)'],
                        ['!hero_ranks', [0], '英雄品阶列表 空(所有)']
                    ], '过滤条件'],
                    ['sort', [
                        ['!fields', [
                        ], '排序字段']
                    ], '排序规则'],
                ],
                'response': [
                    _common.RspHead(),
                    ['page', _common.StreamPage(), '分页信息'],
                    ['!rows', [_common.MarketGoods()], '数据'],
                ]
            },
            {
                'method': 'POST',
                'name': '/api/ingame/asset/hero/mint',
                'desc': '游戏内资产-英雄mint',
                'group': '!AAMarket',
                'url': 'https://market-test.kingsome.cn/api/ingame/asset/hero/mint',
                'headers': _common.JwtHeader,
                'permission': _common.LoginRequired,
                'is_json_params': True,
                'params': [
                    ['to_address', '', '目标address'],
                    ['!hero_uniids', [''], '英雄uniid列表(目前只支持1个)'],
                ],
                'response': [
                    _common.RspHead(),
                    ['trans_id', '', '事务id'],
                    ['trans_req', _common.MFTransactionRequest(), '调用合约参数'],
                ]
            },
            {
                'method': 'GET',
                'name': '/api/user/:account_address',
                'desc': '获取用户信息',
                'group': '!AAMarket',
                'headers': _common.MaybeJwtHeader,
                'url': 'https://market-test.kingsome.cn/api/user/:account_address',
                'params': [
                ],
                'response': [
                    _common.RspHead(),
                    ['nickname', '', '昵称'],
                    ['email', '', 'email(因为隐私问题jwt关联的地址和:account_address相等时才返回email)'],
                    ['contribution_point', 0, '贡献点'],
                    ['gold', '', '金币数'],
                    ['diamond', '', '钻石数'],
                ]
            },
            {
                'method': 'GET',
                'name': '/api/activity/stacking/expected_daily/:account_address',
                'desc': '质押活动-获取每日预期所得',
                'group': '!AAMarket',
                'url': '/api/activity/stacking/expected_daily/:account_address',
                'params': [
                ],
                'response': [
                    _common.RspHead(),
                    ['contribution_point', 0, '贡献点'],
                ]
            },
            {
                'method': 'GET',
                'name': '/api/activity/stacking/history/:account_address',
                'desc': '质押活动-获取每日所得历史记录',
                'group': '!AAMarket',
                'url': '/api/activity/stacking/history/:account_address',
                'params': [
                ],
                'response': [
                    _common.RspHead(),
                    ['!rows', [
                        ['date', 0, '日期utc时间'],
                        ['contribution_point', 0, '贡献点'],
                    ], '数据'],
                ]
            },
            {
                'method': 'GET',
                'name': '/api/activity/diamond/history/:account_address',
                'desc': '钻石-充值/消费记录',
                'group': '!AAMarket',
                'url': '/api/activity/diamond/history/:account_address',
                'params': [
                ],
                'response': [
                    _common.RspHead(),
                    ['!rows', [
                        ['type', 0, '0:充值 1:消费'],
                        ['date', 0, 'utc时间'],
                        ['amount', '', '数量'],
                    ], '数据'],
                ]
            },
            {
                'method': 'GET',
                'name': '/api/activity/bitget/mission/newuser/progress',
                'desc': 'bitget活动-新用户任务进度',
                'group': '!AAMarket',
                'url': '/api/activity/bitget/mission/newuser/progress',
                'params': [
                    ['email', '', '邮箱地址'],
                ],
                'response': [
                    _common.RspHead(),
                    ['!missions', [
                        ['mission_id', 0, '1:玩游戏次数'],
                        ['current', 0, '当前进度'],
                        ['target', 0, '任务目标'],
                    ], '任务列表'],
                ]
            },
            {
                'method': 'GET',
                'name': '/api/activity/cec/contribution/:account_address',
                'desc': 'cec贡献点活动',
                'group': '!AAMarket',
                'url': '/api/activity/cec/contribution/:account_address',
                'params': [
                ],
                'response': [
                    _common.RspHead(),
                    ['info', _common.CecContributionActivity(), 'cec贡献点活动'],
                ]
            },
            {
                'method': 'GET',
                'name': '/api/activity/contribution/history/:account_address',
                'desc': '贡献点活动-获得历史',
                'group': '!AAMarket',
                'url': '/api/activity/contribution/history/:account_address',
                'params': [
                ],
                'response': [
                    _common.RspHead(),
                    ['!rows', [
                        ['type', 0, '1:质押 2:游戏内消费金币'],
                        ['give_time', 0, '获得时间'],
                        ['contribution_point', 0, '贡献点'],
                    ], '数据'],
                ]
            },
            {
                'method': 'GET',
                'name': '/api/chain/txhash/:net_id/:txhash',
                'desc': '返回txhash是否已被服务器确认',
                'group': '!AAMarket',
                'url': '/api/chain/txhash/:net_id/:txhash',
                'params': [
                ],
                'response': [
                    _common.RspHead(),
                    ['confirmed', 0, '是否已确认'],
                ]
            },
            {
                'method': 'POST',
                'name': '/api/chain/event/activity/:account_address',
                'desc': '获取钱包链活动',
                'group': '!AAMarket',
                'url': 'https://market-test.kingsome.cn/api/chain/activity/:account_address',
                'is_json_params': True,
                'request_params': [
                ],
                'params': [
                    ['page_size', 0, '每页大小'],
                    ['cursor', '', '游标 第一页传空'],
                    ['search', [
                        ['name', '', '名字查询 空(所有)'],
                    ], '模糊查询'],
                    ['filter', [
                        ['!item_ids', [0], '道具id列表 空(所有)'],
                    ], '过滤条件'],
                    ['sort', [
                        ['!fields', [
                            ['name', '', '字段名 目前支持的字段'],
                            ['type', 0, '排序方式 -1:倒序 0:默认排序 1:正序'],
                        ], '排序字段']
                    ], '排序规则目前无'],
                ],
                'response': [
                    _common.RspHead(),
                    ['page', _common.StreamPage(), '分页信息'],
                    ['!rows', [_common.ChainActivity()], '数据'],
                ]
            },
            {
                'method': 'GET',
                'name': '/api/recharge/goods',
                'desc': '充值-商品列表',
                'group': '!AAMarket',
                'url': 'https://market-test.kingsome.cn/api/recharge/goods',
                'is_json_params': False,
                'request_params': [
                ],
                'params': [
                    ['net_id', 0, '链id'],
                    ['account_address', '', '钱包地址[可选]'],
                ],
                'response': [
                    _common.RspHead(),
                    ['contribution', 0, '通过充值获得的贡献点数'],
                    ['!rows', [_common.RechargeGoods()], '数据'],
                ]
            },
            {
                'method': 'POST',
                'name': '/api/recharge/buy',
                'desc': '充值-购买',
                'group': '!AAMarket',
                'url': 'https://market-test.kingsome.cn/api/recharge/buy',
                'headers': _common.PassportJwtHeader,
                'is_json_params': True,
                'request_params': [
                ],
                'params': [
                    ['net_id', 0, '链id'],
                    ['goods_id', '', '商品id'],
                    ['goods_num', '', '商品数量'],
                    ['account_address', '', '钱包地址(一般为小狐狸)'],
                    ['currency_address', '', '货币地址'],
                ],
                'response': [
                    _common.RspHead(),
                    ['!calls', [
                        ['trans_id', '', '事务id'],
                        ['trans_req', _common.MFTransactionRequest(), '调用合约参数'],
                    ], '合约调用']
                ]
            },
            {
                'method': 'POST',
                'name': '/api/recharge/buy_with_email',
                'desc': '充值-通过邮件购买',
                'group': '!AAMarket',
                'url': 'https://market-test.kingsome.cn/api/recharge/buy_with_email',
                'headers': _common.JcJwtHeader,
                'is_json_params': True,
                'request_params': [
                ],
                'params': [
                    ['net_id', 0, '链id'],
                    ['goods_id', '', '商品id'],
                    ['goods_num', '', '商品数量'],
                    ['account_address', '', '钱包地址(一般为小狐狸)'],
                    ['currency_address', '', '货币地址'],
                ],
                'response': [
                    _common.RspHead(),
                    ['!calls', [
                        ['trans_id', '', '事务id'],
                        ['trans_req', _common.MFTransactionRequest(), '调用合约参数'],
                    ], '合约调用']
                ]
            },
            {
                'method': 'POST',
                'name': '/api/recharge/history/:net_id/:account_address',
                'desc': '充值-历史',
                'group': '!AAMarket',
                'url': 'https://market-test.kingsome.cn/api/recharge/history/:net_id/:account_address',
                'headers': _common.JwtHeader,
                'is_json_params': True,
                'request_params': [
                ],
                'params': [
                ],
                'response': [
                    _common.RspHead(),
                    ['page', _common.StreamPage(), '分页信息'],
                    ['!rows', [_common.RechargeHistory()], '数据'],
                ]
            },
            {
                'method': 'POST',
                'name': '/api/vip/bind',
                'desc': '绑定vip',
                'group': '!AAMarket',
                'url': 'https://market-test.kingsome.cn/api/vip/bind',
                'is_json_params': True,
                'headers': _common.JcJwtHeader,
                'request_params': [
                ],
                'params': [
                    ['passport_jwt', 0, 'passport_jwt'],
                ],
                'response': [
                    _common.RspHead(),
                ]
            },
            {
                'method': 'GET',
                'name': '/api/vip/info',
                'desc': 'vip绑定信息',
                'group': '!AAMarket',
                'url': 'https://market-test.kingsome.cn/api/vip/info',
                'headers': _common.JcJwtHeader,
                'params': [
                ],
                'response': [
                    _common.RspHead(),
                    ['info', _common.VipInfo(), 'vip信息'],
                ]
            },
            {
                'method': 'GET',
                'name': '/api/vip/details',
                'desc': 'vip等级详情',
                'group': '!AAMarket',
                'url': 'https://market-test.kingsome.cn/api/vip/details',
                'headers': _common.JcJwtHeader,
                'params': [
                ],
                'response': [
                    _common.RspHead(),
                    ['info', _common.VipLevelInfo(), 'vip等级信息'],
                ]
            },
            {
                'method': 'GET',
                'name': '/api/server_switch',
                'desc': '获取功能开关',
                'group': '!AAMarket',
                'url': 'https://market-test.kingsome.cn/api/server_switch',
                'params': [
                ],
                'response': [
                    _common.RspHead(),
                    ['data', [
                        ['heroChain',0,'英雄上链'],
                        ['heroUp',0,'英雄升阶'],
                        ['goldSyn',0,'金币合成'],
                        ['heroPieceSyn',0,'英雄碎片合成'],
                        ['chipPieceSyn',0,'芯片碎片合成'],
                        ['shop',0,'商店'],
                    ], '开关信息(如果没有则默认为未开启)'],
                ]
            },
            {
                'method': 'GET',
                'name': '/api/circuit_ranking',
                'desc': '巡回赛赛季排行',
                'group': '!AAMarket',
                'url': 'https://market-test.kingsome.cn/api/circuit_ranking',
                'params': [
                    ['address', '0', 'address'],
                ],
                'response': [
                    _common.RspHead(),
                    ['!rank_list', [rankingInfo()], '榜'],
                    ['info', circuitInfo(), '信息'],
                ]
            },{
                'name': '/api/circuit_phase_ranking',
                'desc': '巡回赛周期排行',
                'group': '!AAMarket',
                'url': 'https://market-test.kingsome.cn/api/circuit_phase_ranking',
                'params': [
                    ['address', '0', 'address'],
                ],
                'response': [
                    _common.RspHead(),
                    ['!rank_list', [rankingInfo()], '榜'],
                    ['info', circuitInfo(), '信息'],
                ]
            }
        ]
