<?php
require_once('services/AwardService.php');
require_once('services/PropertyChgService.php');

require_once('mt/StarLevel.php');
require_once('mt/BattlePass.php');
require_once('mt/Rank.php');
require_once('mt/RankSeason.php');
require_once('mt/HashRateCommon.php');

require_once('models/RealtimeData.php');
require_once('models/Season.php');
require_once('models/RankingSetRecord.php');

use phpcommon\SqlHelper;

use models\RealtimeData;
use models\Season;
use models\RankingSetRecord;


class DailyRequestController extends BaseAuthedController {
    private $awardService = null;
    private $propertyChgService = null;

    public function _handlePre()
    {
        $this->propertyChgService = new services\PropertyChgService();
        $this->awardService = new services\AwardService();
    }

    /**
     * 通行证的赛季结算
     */
    public function  battlePassSeasonSettlement(){
        $season = RealtimeData::getPassSeason();
        $currSeasonMeta = \mt\BattlePass::getCurrentSeason();
        if ($currSeasonMeta && $currSeasonMeta['id'] > $season) {
            myself()->_getMysql('')->execQuery(
                'UPDATE t_user SET activated=:activated,level=:level,exp=:exp',
                array(
                    "activated" => 0,
                    "level" => 1,
                    "exp" => 0,
                )
            );
            error_log("DailyRequestController : Battle Pass Season ".$season." Settlement Success !" . " Request Time : ". date('Y-M-D h:i:s',time()));
            RealtimeData::setPassSeason($currSeasonMeta['id']);
            $this->_rspData(array('message'=>'通行证结算成功'));
            return;
        }
        $this->_rspOk();
    }

    /**
     * 排位战斗的赛季结算
     */
    public function  battleRankingSeasonSettlement(){
        $currSeasonMeta = mt\RankSeason::getCurrentSeason();
        $lastSeasonMeta = mt\RankSeason::getLastSeason();
        $season = RealtimeData::getRankSeason();

//        if ($currSeasonMeta && $currSeasonMeta['id'] > $season) {
        if (!$currSeasonMeta && $lastSeasonMeta && $lastSeasonMeta['id'] > $season) {
            if ($lastSeasonMeta > 1){
                $SeasonMeta = mt\RankSeason::get($season-1);
                $time = strtotime($SeasonMeta['start_time']);
                $rows = myself()->_getMysql('')->execQuery(
                    'SELECT account_id,`rank`,score FROM t_user ' .
                    'WHERE score_modifytime > :score_modifytime AND score_modifytime <> createtime',
                    array(
                        'score_modifytime' => $time
                    )
                );
            }else{
                $rows = myself()->_getMysql('')->execQuery(
                    'SELECT account_id,`rank`,score FROM t_user ' .
                    'WHERE score_modifytime <> createtime'
                );
            }

            if (count($rows) < 1){
                error_log("resetRankGainAward : No user participates in the ranking");
                die;
            }
            foreach ($rows as $row){
                $currRankMeta = mt\Rank::getRankById($row['rank']);
                if (! $currRankMeta){
                    error_log("resetRankGainAward : rankRank@rankRank.php field rank error" . json_encode($row));
                    continue;
                }
                $dropRankMeta = mt\Rank::getRankById($currRankMeta['next_season']);
                if (! $dropRankMeta){
                    error_log("resetRankGainAward : rankRank@rankRank.php field next_season error" . json_encode($row));
                    continue;
                }
                $award = Season::seasonReward($currRankMeta);
                myself()->switchAccount($row['account_id']);
                myself()->_addItems($award, null, $this->propertyChgService);
                $data = array(
                    'state' => 1,
                    'last_rank' => $row['rank'],
                    'current_rank' => $dropRankMeta['id'],
                    'last_score' => $row['score'],
                    'current_score' => $dropRankMeta['rank_score'],
                    'awards' => json_encode($award),
                );
                RankingSetRecord::upsert($data);
                myself()->_updateUserInfo(array(
                    'last_season_id' => $lastSeasonMeta['id'],
                    'rank' => $dropRankMeta['id'],
                    'score' => $dropRankMeta['rank_score'],
                ));
            }
            error_log("DailyRequestController : Battle Ranking Season ".$season." Settlement Success !" . " Request Time : ". date('Y-M-D h:i:s',time()));
            RealtimeData::setRankSeason($lastSeasonMeta['id']);
            $this->_rspData(array('message'=>'排位结算成功'));
            return;
        }
        $this->_rspOk();
    }


}